@@ -1,6 +1,6 @@
 package SPVM::Net::SSLeay;
 
-our $VERSION = "0.026";
+our $VERSION = "0.027";
 
 1;
 
@@ -72,10 +72,6 @@ OpenSSL 1.1.1
 
 =item * L<Net::SSLeay::DER|SPVM::Net::SSLeay::DER>
 
-=item * L<Net::SSLeay::DH|SPVM::Net::SSLeay::DH>
-
-=item * L<Net::SSLeay::EC_KEY|SPVM::Net::SSLeay::EC_KEY>
-
 =item * L<Net::SSLeay::ERR|SPVM::Net::SSLeay::ERR>
 
 =item * L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error>
@@ -108,8 +104,6 @@ OpenSSL 1.1.1
 
 =item * L<Net::SSLeay::SSL_METHOD|SPVM::Net::SSLeay::SSL_METHOD>
 
-=item * L<Net::SSLeay::SSL_SESSION|SPVM::Net::SSLeay::SSL_SESSION>
-
 =item * L<Net::SSLeay::Util|SPVM::Net::SSLeay::Util>
 
 =item * L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509>
@@ -148,12 +142,6 @@ See source codes of L<IO::Socket::SSL|https://metacpan.org/pod/SPVM::IO::Socket:
 
 =head1 Fields
 
-=head2 ssl_ctx
-
-C<has ssl_ctx : L<Net::SSLeay::SSL_CTX|SPVM::Net::SSLeay::SSL_CTX>;>
-
-A L<Net::SSLeay::SSL_CTX|SPVM::Net::SSLeay::SSL_CTX> object.
-
 =head2 operation_error
 
 C<has operation_error : ro int;>
@@ -174,11 +162,11 @@ Exceptions:
 
 If SSL_new failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
 
-=head2 load_error_strings
+=head2 alert_desc_string_long
 
-C<static method load_error_strings : void ();>
+C<static method alert_desc_string_long : string ($value : int);>
 
-Calls native L<SSL_load_error_strings|https://docs.openssl.org/3.0/man3/ERR_load_crypto_strings/> function.
+Calls native L<SSL_alert_desc_string_long|https://docs.openssl.org/1.1.1/man3/SSL_alert_type_string/> function given the pointer value of the instance, $type, and returns its return value.
 
 =head2 load_client_CA_file
 
@@ -198,15 +186,29 @@ If load_client_CA_file failed, an exception is thrown with C<eval_error_id> set
 
 =head1 Instance Methods
 
-=head2 set_fd
+=head2 version
 
-C<method set_fd : int ($fd : int);>
+C<native method version : int ();>
 
-Calls native L<SSL_set_fd|https://docs.openssl.org/master/man3/SSL_set_fd/> function given the pointer value of the instance, $fd, and returns its return value.
+Calls native L<version|https://docs.openssl.org/master/man3/SSL_get_version> function, and returns its return value.
 
-Exceptions:
+=head2 get_mode
 
-If SSL_set_fd failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
+C<method get_mode : long ();>
+
+Calls native L<get_mode|https://docs.openssl.org/1.1.1/man3/SSL_CTX_set_mode/> function, and returns its return value.
+
+=head2 set_mode
+
+C<method set_mode : long ($mode : long);>
+
+Calls native L<SSL_set_mode|https://docs.openssl.org/1.1.1/man3/SSL_CTX_set_mode/> function given the pointer value of the instance, $mode, and returns its return value.
+
+=head2 clear_mode
+
+C<method clear_mode : long ($mode : long);>
+
+Calls native L<SSL_clear_mode|https://docs.openssl.org/1.1.1/man3/SSL_CTX_set_mode/> function given the pointer value of the instance, $mode, and returns its return value.
 
 =head2 set_tlsext_host_name
 
@@ -220,6 +222,34 @@ The host name $name must be defined. Otherwise an exception is thrown.
 
 If SSL_set_tlsext_host_name failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
 
+=head2 get_servername
+
+C<method get_servername : string ($type : int);>
+
+Calls native L<SSL_get_servername|https://docs.openssl.org/master/man3/SSL_CTX_set_tlsext_servername_callback> function given the pointer value of the instance, $type, and returns its return value.
+
+=head2 get_SSL_CTX
+
+C<method get_SSL_CTX : L<Net::SSLeay::SSL_CTX|SPVM::Net::SSLeay::SSL_CTX> ();>
+
+Calls native L<SSL_get_SSL_CTX|https://docs.openssl.org/3.4/man3/SSL_get_SSL_CTX/> function given the pointer value of the instance, creates a new L<Net::SSLeay::SSL_CTX|SPVM::Net::SSLeay::SSL_CTX> object, sets the pointer value of the new object to the return value of the native function, sets C<no_free> flag of the new object to 1, creates a reference from the new object to the instance, and returns the new object.
+
+=head2 set_SSL_CTX
+
+C<method set_SSL_CTX : Net::SSLeay::SSL_CTX ($ctx : Net::SSLeay::SSL_CTX);>
+
+Calls native L<SSL_set_SSL_CTX|https://docs.openssl.org/3.4/man3/SSL_get_SSL_CTX/> function given the pointer value of the instance, creates a new L<Net::SSLeay::SSL_CTX|SPVM::Net::SSLeay::SSL_CTX> object, sets the pointer value of the new object to the return value of the native function, sets C<no_free> flag of the new object to 1, creates a reference from the new object to the instance, and returns the new object.
+
+=head2 set_fd
+
+C<method set_fd : int ($fd : int);>
+
+Calls native L<SSL_set_fd|https://docs.openssl.org/master/man3/SSL_set_fd/> function given the pointer value of the instance, $fd, and returns its return value.
+
+Exceptions:
+
+If SSL_set_fd failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
+
 =head2 connect
 
 C<method connect : int ();>
@@ -240,16 +270,6 @@ Exceptions:
 
 If SSL_accept failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class and with L</"operation_error"> field set to the return vlaue of L<SSL_get_error|https://docs.openssl.org/1.1.1/man3/SSL_get_error/> function given the return value of L<SSL_accept|https://docs.openssl.org/1.0.2/man3/SSL_accept/> function.
 
-=head2 shutdown
-
-C<method shutdown : int ();>
-
-Calls native L<SSL_shutdown|https://docs.openssl.org/master/man3/SSL_shutdown/> function, and returns its return value.
-
-Exceptions:
-
-If SSL_shutdown failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class and with L</"operation_error"> field set to the return vlaue of L<SSL_get_error|https://docs.openssl.org/1.1.1/man3/SSL_get_error/> function given the return value of L<SSL_shutdown|https://docs.openssl.org/1.0.2/man3/SSL_shutdown/> function.
-
 =head2 read
 
 C<method read : int ($buf : mutable string, $num : int = -1, $offset : int = 0);>
@@ -266,22 +286,6 @@ The offset $offset + $num must be lower than or equal to the length of the buffe
 
 If SSL_read failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class and with L</"operation_error"> field set to the return vlaue of L<SSL_get_error|https://docs.openssl.org/1.1.1/man3/SSL_get_error/> function given the return value of L<SSL_read|https://docs.openssl.org/1.0.2/man3/SSL_read/> function.
 
-=head2 peek
-
-C<method peek : int ($buf : mutable string, $num : int = -1, $offset : int = 0);>
-
-Calls native L<SSL_peek|https://docs.openssl.org/1.1.1/man3/SSL_peek/> function given the pointer value of the instance, $buf at the offset $offset, $num, and returns its return value.
-
-Exceptions:
-
-The buffer $buf must be defined. Otherwise an exception is thrown.
-
-The offset $offset must be greater than or equal to 0. Otherwise an exception is thrown.
-
-The offset $offset + $num must be lower than or equal to the length of the buffer $buf. Otherwise an exception is thrown.
-
-If SSL_peek failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class and with L</"operation_error"> field set to the return vlaue of L<SSL_get_error|https://docs.openssl.org/1.1.1/man3/SSL_get_error/> function given the return value of L<SSL_peek|https://docs.openssl.org/1.0.2/man3/SSL_peek/> function.
-
 =head2 write
 
 C<method write : int ($buf : string, $num : int = -1, $offset : int = 0);>
@@ -298,73 +302,15 @@ The offset $offset + $num must be lower than or equal to the length of the buffe
 
 If SSL_write failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class and with L</"operation_error"> field set to the return vlaue of L<SSL_get_error|https://docs.openssl.org/1.1.1/man3/SSL_get_error/> function given the return value of L<SSL_write|https://docs.openssl.org/1.0.2/man3/SSL_write/> function.
 
-=head2 get_servername
+=head2 shutdown
 
-C<method get_servername : string ($type : int);>
+C<method shutdown : int ();>
 
-Calls native L<SSL_get_servername|https://docs.openssl.org/master/man3/SSL_CTX_set_tlsext_servername_callback> function given the pointer value of the instance, $type, and returns its return value.
-
-=head2 set_tlsext_status_type
-
-C<method set_tlsext_status_type : long  ($type : int);>
-
-Calls native L<SSL_set_tlsext_status_type|https://docs.openssl.org/1.0.2/man3/SSL_CTX_set_tlsext_status_cb> function given the pointer value of the instance, $type, and returns its return value.
+Calls native L<SSL_shutdown|https://docs.openssl.org/master/man3/SSL_shutdown/> function, and returns its return value.
 
 Exceptions:
 
-If SSL_set_tlsext_status_type failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
-
-=head2 alert_desc_string_long
-
-C<method alert_desc_string_long : string  ($type : int);>
-
-Calls native L<SSL_alert_desc_string_long|https://docs.openssl.org/1.1.1/man3/SSL_alert_type_string/> function given the pointer value of the instance, $type, and returns its return value.
-
-=head2 set_mode
-
-C<method set_mode : long ($mode : long);>
-
-Calls native L<SSL_set_mode|https://docs.openssl.org/1.1.1/man3/SSL_CTX_set_mode/> function given the pointer value of the instance, $mode, and returns its return value.
-
-=head2 clear_mode
-
-C<method clear_mode : long ($mode : long);>
-
-Calls native L<SSL_clear_mode|https://docs.openssl.org/1.1.1/man3/SSL_CTX_set_mode/> function given the pointer value of the instance, $mode, and returns its return value.
-
-=head2 get_mode
-
-C<method get_mode : long ();>
-
-Calls native L<get_mode|https://docs.openssl.org/1.1.1/man3/SSL_CTX_set_mode/> function, and returns its return value.
-
-=head2 version
-
-C<native method version : int ();>
-
-Calls native L<version|https://docs.openssl.org/master/man3/SSL_get_version> function, and returns its return value.
-
-=head2 session_reused
-
-C<native method session_reused : int ();>
-  
-Calls native L<SSL_session_reused|https://docs.openssl.org/1.1.1/man3/SSL_session_reused/> function, and returns its return value.
-
-=head2 get_cipher
-
-C<method get_cipher : string ();>
-
-Calls native L<SSL_get_cipher|https://docs.openssl.org/1.0.2/man3/SSL_get_current_cipher/> function, and returns its return value.
-
-=head2 get_peer_certificate
-
-C<method get_peer_certificate : L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> ();>
-
-Calls native L<SSL_get_peer_certificate|https://docs.openssl.org/master/man3/SSL_get_peer_certificate> function.
-
-If the return value of the native function is NULL, returns undef.
-
-Otherwise, creates a new L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> object, sets the pointer value of the new object to the return value of the native function, and returns the new object.
+If SSL_shutdown failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class and with L</"operation_error"> field set to the return vlaue of L<SSL_get_error|https://docs.openssl.org/1.1.1/man3/SSL_get_error/> function given the return value of L<SSL_shutdown|https://docs.openssl.org/1.0.2/man3/SSL_shutdown/> function.
 
 =head2 get_shutdown
 
@@ -372,33 +318,11 @@ C<method get_shutdown : int ();>
 
 Calls native L<SSL_get_shutdown|https://docs.openssl.org/master/man3/SSL_set_shutdown/> function, and returns its return value.
 
-=head2 pending
+=head2 get_cipher
 
-C<method pending : int ();>
+C<method get_cipher : string ();>
 
-Calls native L<SSL_pending|https://docs.openssl.org/master/man3/SSL_pending/> function, and returns its return value.
-
-=head2 get1_session
-
-C<method get1_session : L<Net::SSLeay::SSL_SESSION|SPVM::Net::SSLeay::SSL_SESSION> ();>
-
-Calls native L<SSL_get1_session|https://docs.openssl.org/1.1.1/man3/SSL_get_session> function.
-
-If the return value of the native function is NULL, returns undef.
-
-Otherwise, creates a new L<Net::SSLeay::SSL_SESSION|SPVM::Net::SSLeay::SSL_SESSION> object, sets the pointer value of the new object to the return value of the native function, and returns the new object.
-
-=head2 set_session
-
-C<method set_session : int ($session : Net::SSLeay::SSL_SESSION);>
-
-Calls native L<SSL_set_session|https://docs.openssl.org/1.1.1/man3/SSL_set_session> function given the pointer value of $session, sets L</"ssl_session"> field to $ssl.
-
-If this method succeeds, C<no_free> flag of $session is set to 1.
-
-Exceptions:
-
-If SSL_set_session failed, an exception is thrown with C<eval_error_id> set to the basic type ID of L<Net::SSLeay::Error|SPVM::Net::SSLeay::Error> class.
+Calls native L<SSL_get_cipher|https://docs.openssl.org/1.0.2/man3/SSL_get_current_cipher/> function, and returns its return value.
 
 =head2 get_certificate
 
@@ -412,17 +336,15 @@ Otherwise, creates a new L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> object, se
 
 C<no_free> flag of the new object is set to 1.
 
-=head2 get0_next_proto_negotiated
+=head2 get_peer_certificate
 
-C<method get0_next_proto_negotiated : void ($data_ref : string[], $len_ref : int*);>
+C<method get_peer_certificate : L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> ();>
 
-Calls native L<SSL_get0_next_proto_negotiated|https://docs.openssl.org/master/man3/SSL_get_certificate> function given the pointer value of the instance, $data_ref, $len_ref.
+Calls native L<SSL_get_peer_certificate|https://docs.openssl.org/master/man3/SSL_get_peer_certificate> function.
 
-=head2 get0_next_proto_negotiated_return_string
+If the return value of the native function is NULL, returns undef.
 
-C<method get0_next_proto_negotiated_return_string : string ()>
-
-Calls L</"get0_next_proto_negotiated"> method given appropriate arguments, and returns the output string.
+Otherwise, creates a new L<Net::SSLeay::X509|SPVM::Net::SSLeay::X509> object, sets the pointer value of the new object to the return value of the native function, and returns the new object.
 
 =head2 get0_alpn_selected
 
@@ -448,12 +370,6 @@ Ohterwise, converts its return value to the array of L<Net::SSLeay::X509|SPVM::N
 
 method get_SSL_CTX : Net::SSLeay::SSL_CTX ();
 
-=head2 get_SSL_CTX
-
-C<method get_SSL_CTX : Net::SSLeay::SSL_CTX ();>
-
-Returns the value of L</"ssl_ctx"> field.
-
 =head2 set_msg_callback
 
 C<method set_msg_callback : void ($cb : L<Net::SSLeay::Callback::Msg|SPVM::Net::SSLeay::Callback::Msg>, $arg : object = undef);>
@@ -466,12 +382,6 @@ C<static method dump_peer_certificate : string ();>
 
 Returns the same output of Perl's L<Net::SSLeay#dump_peer_certificate|https://metacpan.org/dist/Net-SSLeay/view/lib/Net/SSLeay.pod#Convenience-routines> function.
 
-=head2 get_tlsext_status_type
-
-C<method get_tlsext_status_type : long ();>
-
-Calls native L<SSL_get_tlsext_status_type|https://docs.openssl.org/master/man3/SSL_CTX_set_tlsext_status_cb> function given the pointer value of the instance, and returns its return value.
-
 =head2 DESTROY
 
 C<method DESTROY : void ();>
