@@ -6,10 +6,8 @@
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
-
 static const char* FILE_NAME = "Net/SSLeay/X509_STORE_CTX.c";
 
-// Class Methods
 int32_t SPVM__Net__SSLeay__X509_STORE_CTX__get1_issuer(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t error_id = 0;
@@ -20,14 +18,8 @@ int32_t SPVM__Net__SSLeay__X509_STORE_CTX__get1_issuer(SPVM_ENV* env, SPVM_VALUE
   
   void* obj_x = stack[2].oval;
   
-  if (!obj_issuer_ref) {
-    return env->die(env, stack, "The output array of the Net::SSLeay::X509 $issuer_ref must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  int32_t issuer_ref_length = env->length(env, stack, obj_issuer_ref);
-  
-  if (!(issuer_ref_length == 1)) {
-    return env->die(env, stack, "The length of $issuer_ref must be 1.", __func__, FILE_NAME, __LINE__);
+  if (!(obj_issuer_ref && env->length(env, stack, obj_issuer_ref) == 1)) {
+    return env->die(env, stack, "The output array of the Net::SSLeay::X509 $issuer_ref must be a 1-length array.", __func__, FILE_NAME, __LINE__);
   }
   
   if (!obj_ctx) {
@@ -78,7 +70,6 @@ int32_t SPVM__Net__SSLeay__X509_STORE_CTX__get1_issuer(SPVM_ENV* env, SPVM_VALUE
   return 0;
 }
 
-// Instance Methods
 int32_t SPVM__Net__SSLeay__X509_STORE_CTX__set_error(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t error_id = 0;
@@ -137,13 +128,13 @@ int32_t SPVM__Net__SSLeay__X509_STORE_CTX__get_current_cert(SPVM_ENV* env, SPVM_
   void* obj_x509 = NULL;
   
   if (x509) {
+    X509_up_ref(x509);
     void* obj_address_x509 = env->new_pointer_object_by_name(env, stack, "Address", x509, &error_id, __func__, FILE_NAME, __LINE__);
     if (error_id) { return error_id; }
     stack[0].oval = obj_address_x509;
     env->call_class_method_by_name(env, stack, "Net::SSLeay::X509", "new_with_pointer", 1, &error_id, __func__, FILE_NAME, __LINE__);  
     if (error_id) { return error_id; }
     obj_x509 = stack[0].oval;
-    env->set_no_free(env, stack, obj_x509, 1);
   }
   
   stack[0].oval = obj_x509;
@@ -151,68 +142,6 @@ int32_t SPVM__Net__SSLeay__X509_STORE_CTX__get_current_cert(SPVM_ENV* env, SPVM_
   return 0;
 }
 
-int32_t SPVM__Net__SSLeay__X509_STORE_CTX__Init(SPVM_ENV* env, SPVM_VALUE* stack) {
-  
-  int32_t error_id = 0;
-  
-  void* obj_self = stack[0].oval;
-  
-  void* obj_trust_store = stack[1].oval;
-  
-  void* obj_target = stack[2].oval;
-  
-  void* obj_untrusted_array = stack[3].oval;
-  
-  X509_STORE_CTX* self = env->get_pointer(env, stack, obj_self);
-  
-  if (!obj_trust_store) {
-    return env->die(env, stack, "The X509_STORE object $trust_store must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  X509_STORE* trust_store = env->get_pointer(env, stack, obj_trust_store);
-  
-  if (!obj_target) {
-    return env->die(env, stack, "The X509 object $target must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  X509* target = env->get_pointer(env, stack, obj_target);
-  
-  if (!obj_untrusted_array) {
-    return env->die(env, stack, "The X509 array $untrusted_array must be defined.", __func__, FILE_NAME, __LINE__);
-  }
-  
-  STACK_OF(X509)* sk_X509 = sk_X509_new_null();
-  
-  int32_t list_length = env->length(env, stack, obj_untrusted_array);
-  
-  for (int32_t i = 0; i < list_length; i++) {
-    void* obj_X509 = env->get_elem_object(env, stack, obj_untrusted_array, i);
-    X509* X509 = env->get_pointer(env, stack, obj_X509);
-    sk_X509_push(sk_X509, X509);
-  }
-  
-  int32_t status = X509_STORE_CTX_init(self, trust_store, target, sk_X509);
-  
-  if (!(status == 1)) {
-    int64_t ssl_error = ERR_peek_last_error();
-    
-    char* ssl_error_string = env->get_stack_tmp_buffer(env, stack);
-    ERR_error_string_n(ssl_error, ssl_error_string, SPVM_NATIVE_C_STACK_TMP_BUFFER_SIZE);
-    
-    env->die(env, stack, "[OpenSSL Error]X509_STORE_CTX_init failed:%s.", ssl_error_string, __func__, FILE_NAME, __LINE__);
-    
-    int32_t tmp_error_id = env->get_basic_type_id_by_name(env, stack, "Net::SSLeay::Error", &error_id, __func__, FILE_NAME, __LINE__);
-    if (error_id) { return error_id; }
-    error_id = tmp_error_id;
-    
-    return error_id;
-  }
-  
-  stack[0].ival = status;
-  
-  return 0;
-}
-
 int32_t SPVM__Net__SSLeay__X509_STORE_CTX__DESTROY(SPVM_ENV* env, SPVM_VALUE* stack) {
   
   int32_t error_id = 0;
